@@ -67,9 +67,11 @@ mcnr_family <- function(family, cmdstan){
   gaussian_list <- c("identity")
   binomial_list <- c("logit","log","identity","probit")
   bernoulli_list <- c("logit","log","identity","probit")
+  quantile_list <- quantile_scaled_list <- c("identity","log","logit","probit","inverse")
   poisson_list <- c("log")
   gamma_list <- c("identity","inverse","log")
   beta_list <- c("logit")
+  if(f1 == "quantile_scaled")f1 <- "quantile"
   type <- which(get(paste0(f1,"_list"))==link)
   if(length(type)==0)stop("link not supported for this family")
   if(cmdstan){
