@@ -97,6 +97,7 @@ public:
   std::pair<double,double>  current_likelihood_values();
   std::pair<double,double>  u_diagnostic();
   void            reset_fn_counter();
+  void            set_quantile(const double& q);
   // functions to optimise
   double          log_likelihood_beta(const dblvec &beta);
   double          log_likelihood_beta_with_gradient(const VectorXd &beta, VectorXd& g);
@@ -115,6 +116,7 @@ protected:
   dblvec    lower_bound_theta;
   dblvec    upper_bound_theta; // bounds for beta
   bool      beta_bounded = false;
+  double    quantile = 0.5;
   
   // functions
   void            calculate_var_par();
@@ -459,6 +461,13 @@ inline void glmmr::ModelOptim<modeltype>::reset_fn_counter()
   fn_counter.second = 0;
 }
 
+template<typename modeltype>
+inline void glmmr::ModelOptim<modeltype>::set_quantile(const double& q)
+{
+  if(q <= 0 || q >= 1)throw std::runtime_error("q !in [0,1]");
+  quantile = q;
+}
+
 template<typename modeltype>
 inline double glmmr::ModelOptim<modeltype>::ll_diff_variance(bool beta, bool theta)
 {
@@ -913,31 +922,30 @@ inline double glmmr::ModelOptim<modeltype>::log_likelihood(bool beta) {
   
   if(model.weighted){
     if(model.family.family==Fam::gaussian){
-      for(int j= 0; j< re.zu_.cols() ; j++){
 #pragma omp parallel for
+      for(int j= 0; j< re.zu_.cols() ; j++){
         for(int i = 0; i<model.n(); i++){
           ll_current(j,llcol ) += glmmr::maths::log_likelihood(model.data.y(i),xb(i) + re.zu_(i,j),
                                              model.data.variance(i)/model.data.weights(i),
-                                             model.family.family,model.family.link);
+                                             model.family);
         }
       }
     } else {
-      for(int j=0; j< re.zu_.cols() ; j++){
 #pragma omp parallel for
+      for(int j=0; j< re.zu_.cols() ; j++){
         for(int i = 0; i<model.n(); i++){
           ll_current(j,llcol) += model.data.weights(i)*glmmr::maths::log_likelihood(model.data.y(i),xb(i) + re.zu_(i,j),
-                                   model.data.variance(i),model.family.family,model.family.link);
+                                   model.data.variance(i),model.family);
         }
       }
       ll_current.col(llcol) *= model.data.weights.sum()/model.n();
     }
   } else {
-    for(int j= 0; j< re.zu_.cols() ; j++){
 #pragma omp parallel for
+    for(int j= 0; j< re.zu_.cols() ; j++){
       for(int i = 0; i<model.n(); i++){
         ll_current(j,llcol) += glmmr::maths::log_likelihood(model.data.y(i),xb(i) + re.zu_(i,j),
-                                           model.data.variance(i),model.family.family,
-                                           model.family.link);
+                                           model.data.variance(i),model.family);
       }
     }
   }
@@ -982,7 +990,7 @@ inline dblvec glmmr::ModelOptim<modeltype>::get_start_values(bool beta, bool the
 template<typename modeltype>
 inline void glmmr::ModelOptim<modeltype>::set_bound(const dblvec& bound, bool lower)
 {
-  if(bound.size()!=P())throw std::runtime_error("Bound not equal to number of parameters");
+  if(static_cast<int>(bound.size())!=P())throw std::runtime_error("Bound not equal to number of parameters");
   if(lower){
     if(lower_bound.size() != bound.size())lower_bound.resize(P());
     lower_bound = bound; 
@@ -1172,17 +1180,29 @@ inline void glmmr::ModelOptim<modeltype>::update_var_par(const ArrayXd& v){
 
 template<typename modeltype>
 inline void glmmr::ModelOptim<modeltype>::calculate_var_par(){
-  if(model.family.family==Fam::gaussian){
+  if(model.family.family==Fam::gaussian || model.family.family==Fam::quantile_scaled){
     // revise this for beta and Gamma re residuals
     int niter = re.u(false).cols();
     ArrayXd sigmas(niter);
+    sigmas.setZero();
     MatrixXd zd = matrix.linpred();
 #pragma omp parallel for
     for(int i = 0; i < niter; ++i){
       VectorXd zdu = glmmr::maths::mod_inv_func(zd.col(i), model.family.link);
       ArrayXd resid = (model.data.y - zdu);
       resid *= model.data.weights.sqrt();
-      sigmas(i) = (resid - resid.mean()).square().sum()/(resid.size()-1);
+      if(model.family.family==Fam::gaussian){
+        sigmas(i) = (resid - resid.mean()).square().sum()/(resid.size()-1.0);
+      } else {
+        for(int j = 0; j < resid.size(); j++){
+          if(resid(j) < 0){
+            sigmas(i) += resid(j)*(model.family.quantile - 1.0);
+          } else {
+            sigmas(i) += resid(j)*model.family.quantile;
+          }
+        }
+        sigmas(i) *= 1.0/resid.size();
+      }
     }
     update_var_par(sigmas.mean());
   }
