@@ -55,24 +55,22 @@ void Model__set_bound(SEXP xp, SEXP bound_, bool beta = true, bool lower = true,
 }
 
 // [[Rcpp::export]]
-void Model__print_instructions(SEXP xp, bool linpred, bool loglik, int type = 0){
+void Model__print_instructions(SEXP xp, int type = 0){
   glmmrType model(xp,static_cast<Type>(type));
   auto functor1 = overloaded {
     [](int) {}, 
     [](auto ptr){ptr->model.linear_predictor.calc.print_instructions();}
   };
-  auto functor3 = overloaded {
-    [](int) {}, 
-    [](auto ptr){ptr->model.calc.print_instructions();}
-  };
-  if(linpred){
-    Rcpp::Rcout << "\nLINEAR PREDICTOR:\n";
-    std::visit(functor1,model.ptr);
-  }
-  if(loglik){
-    Rcpp::Rcout << "\nLOG-LIKELIHOOD:\n";
-    std::visit(functor3,model.ptr);
-  }
+  // auto functor3 = overloaded {
+  //   [](int) {}, 
+  //   [](auto ptr){ptr->model.calc.print_instructions();}
+  // };
+  Rcpp::Rcout << "\nLINEAR PREDICTOR:\n";
+  std::visit(functor1,model.ptr);
+  // if(loglik){
+  //   Rcpp::Rcout << "\nLOG-LIKELIHOOD:\n";
+  //   std::visit(functor3,model.ptr);
+  // }
 }
 
 // [[Rcpp::export]]
